<?php
require __DIR__ . '/includes/application_top.php';
require_once __DIR__ . '/vendor/davefx/phplot/phplot/phplot.php';

$_SESSION['customers_status']['customers_status_id'] === DEFAULT_CUSTOMERS_STATUS_ID_ADMIN or die('go away. please.');

$twentyFourHoursAgo = (new \DateTime('24 hours ago'))->getTimestamp();

$profilingLogfile = DIR_FS_CATALOG . '/logfiles/gethubpayment-profiling-' . LogControl::get_secure_token() . '.log';
$logData = file($profilingLogfile);

$plotData = [];
$plotData24h = [];
$statistics = [];
$statistics24h = [];
$durationSum = 0;
$durationSum24h = 0;
$logData24h = [];
foreach($logData as $logLine) {
    list($ts, $tsRaw, $session, $hubSession, $customerId, $duration) = explode(',', trim($logLine));
    $durationSum += (float)$duration;
    $plotData[] = [$ts, (float)$tsRaw, (float)$duration];
    $class = ((int)floor((float)$duration)) + 1 . 's';
    if ($statistics[$class] === null) {
    	$statistics[$class] = [
    		$class,
		    1
	    ];
    } else {
    	$statistics[$class][1]++;
    }
    if ((float)$tsRaw >= $twentyFourHoursAgo) {
    	$logData24h[] = $logLine;
        $durationSum24h += (float)$duration;
        $plotData24h[] = [$ts, (float)$tsRaw, (float)$duration];
        $class = ((int)floor((float)$duration)) + 1 . 's';
        if ($statistics24h[$class] === null) {
            $statistics24h[$class] = [
                $class,
                1
            ];
        } else {
            $statistics24h[$class][1]++;
        }
    }
}
uksort($statistics, 'strnatcmp');
uksort($statistics24h, 'strnatcmp');

$plot = new PHPlot(1600, 600);
$plot->SetPrintImage(false);
$plot->SetDataType('data-data');
$plot->SetDataValues($plotData);
$plot->SetXLabelAngle(90);
$plot->DrawGraph();
$imgData = $plot->EncodeImage('dataurl');

$plot2 = new PHPlot(1000, 400);
$plot2->SetPrintImage(false);
$plot2->SetDataType('text-data');
$plot2->SetDataValues(array_values($statistics));
$plot2->SetPlotType('bars');
$plot2->DrawGraph();
$distImgData = $plot2->EncodeImage('dataurl');

if (!empty($plotData24h)) {
    $plot24h = new PHPlot(800, 400);
    $plot24h->SetPrintImage(false);
    $plot24h->SetDataType('data-data');
    $plot24h->SetDataValues($plotData24h);
    $plot24h->SetXLabelAngle(90);
    $plot24h->DrawGraph();
    $imgData24h = $plot24h->EncodeImage('dataurl');
    
    $plot2_24h = new PHPlot(800, 400);
    $plot2_24h->SetPrintImage(false);
    $plot2_24h->SetDataType('text-data');
    $plot2_24h->SetDataValues(array_values($statistics24h));
    $plot2_24h->SetPlotType('bars');
    $plot2_24h->DrawGraph();
    $distImgData24h = $plot2_24h->EncodeImage('dataurl');
}

?>
<!DOCTYPE html>
<html>
    <head>
        <title>Statistik Hub-Zugriffe</title>
	    <style>
		    p.graph {
			    text-align: center;
		    }
		    table.statistics {
			    width: 20em;
			    border-collapse: collapse;
			    margin: 2em auto;
		    }
		    table.statistics th {
			    background: #eee;
		    }
		    table.statistics td, table.statistics th {
			    border-right: 1px solid #888;
			    padding: .2ex .5em;
		    }
		    table.statistics td.right {
			    text-align: right;
		    }
		    footer {
			    font-size: .8em;
			    border-top: 2px solid #000000;
		    }
	    </style>
    </head>
    <body>
	    <h1>Gesamtstatistik</h1>
	    <p class="graph">
		    <img src="<?= $imgData ?>">
	    </p>
	    <p class="graph">
		    <img src="<?= $distImgData ?>">
	    </p>
	    <table class="statistics">
		    <tr>
			    <th>Duration</th>
			    <th>Anzahl</th>
			    <th>Anteil</th>
			    <th>kumulativ</th>
		    </tr>
		    <?php foreach($statistics as $key => $statLine): ?>
			    <tr>
				    <td>bis <?= $key ?></td>
				    <td class="right"><?= $statLine[1] ?></td>
				    <td class="right"><?= number_format(100 * $statLine[1]/count($logData), 1, '.', '') ?>%</td>
				    <td class="right"><?php $kumulativ += 100 * $statLine[1]/count($logData); echo number_format($kumulativ, 1, '.', ''); ?>%</td>
			    </tr>
		    <?php endforeach?>
		    <tr>
			    <th>∅ <?= number_format($durationSum / count($logData), 3, '.', '') ?>s</th>
			    <th><?= count($logData) ?></th>
			    <th>&nbsp;</th>
			    <th>&nbsp;</th>
		    </tr>
	    </table>
	    <pre>
		    <?php //var_dump($statistics); ?>
	    </pre>
	    
		<?php if (!empty($logData24h)): ?>
		    <h1>Letzte 24 Stunden</h1>
		    <p class="graph">
			    <img src="<?= $imgData24h ?>">
			    <img src="<?= $distImgData24h ?>">
		    </p>
		    <table class="statistics">
			    <tr>
				    <th>Duration</th>
				    <th>Anzahl</th>
				    <th>Anteil</th>
				    <th>kumulativ</th>
			    </tr>
	            <?php foreach($statistics24h as $key => $statLine): ?>
				    <tr>
					    <td>bis <?= $key ?></td>
					    <td class="right"><?= $statLine[1] ?></td>
					    <td class="right"><?= number_format(100 * $statLine[1]/count($logData24h), 1, '.', '') ?>%</td>
					    <td class="right"><?php $kumulativ24h += 100 * $statLine[1]/count($logData24h); echo number_format($kumulativ24h, 1, '.', ''); ?>%</td>
				    </tr>
	            <?php endforeach?>
			    <tr>
				    <th>∅ <?= number_format($durationSum24h / count($logData24h), 3, '.', '') ?>s</th>
				    <th><?= count($logData24h) ?></th>
				    <th>&nbsp;</th>
				    <th>&nbsp;</th>
			    </tr>
		    </table>
        <?php else: ?>
			<h1>Keine Zugriffe in den letzten 24 Stunden</h1>
	    <?php endif ?>
	    
	    <footer>
		    Serverzeit: <?= (new \DateTime())->format('c') ?>
	    </footer>
    </body>
</html>
